from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session

from app import crud, models, schemas
from app.api import deps

router = APIRouter()


@router.get("/", response_model=List[schemas.Product])
def read_products(
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Retrieve products.
    """
    products = crud.product.get_multi_by_owner(
        db, owner_id=current_user.id, skip=skip, limit=limit
    )
    return products


@router.post("/", response_model=schemas.Product)
def create_product(
    *,
    db: Session = Depends(deps.get_db),
    product_in: schemas.ProductCreate,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Create new product.
    """
    # Check if product with same SKU already exists
    product = crud.product.get_by_sku(db, sku=product_in.sku)
    if product:
        raise HTTPException(
            status_code=400,
            detail="A product with this SKU already exists.",
        )
    
    product = crud.product.create_with_owner(
        db, obj_in=product_in, owner_id=current_user.id
    )
    return product


@router.get("/{id}", response_model=schemas.Product)
def read_product(
    *,
    db: Session = Depends(deps.get_db),
    id: str,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Get product by ID.
    """
    product = crud.product.get(db, id=id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    if product.owner_id != current_user.id and not current_user.is_superuser:
        raise HTTPException(status_code=403, detail="Not enough permissions")
    return product


@router.put("/{id}", response_model=schemas.Product)
def update_product(
    *,
    db: Session = Depends(deps.get_db),
    id: str,
    product_in: schemas.ProductUpdate,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Update a product.
    """
    product = crud.product.get(db, id=id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    if product.owner_id != current_user.id and not current_user.is_superuser:
        raise HTTPException(status_code=403, detail="Not enough permissions")
    product = crud.product.update(db, db_obj=product, obj_in=product_in)
    return product


@router.delete("/{id}", status_code=204, response_model=None)
def delete_product(
    *,
    db: Session = Depends(deps.get_db),
    id: str,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Delete a product.
    """
    product = crud.product.get(db, id=id)
    if not product:
        raise HTTPException(status_code=404, detail="Product not found")
    if product.owner_id != current_user.id and not current_user.is_superuser:
        raise HTTPException(status_code=403, detail="Not enough permissions")
    crud.product.remove(db, id=id)
    return None


@router.get("/by-category/{category_id}", response_model=List[schemas.Product])
def read_products_by_category(
    *,
    db: Session = Depends(deps.get_db),
    category_id: str,
    skip: int = 0,
    limit: int = 100,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Retrieve products by category.
    """
    products = crud.product.get_multi_by_category(
        db, category_id=category_id, skip=skip, limit=limit
    )
    # Filter by owner
    return [p for p in products if p.owner_id == current_user.id]


@router.get("/by-supplier/{supplier_id}", response_model=List[schemas.Product])
def read_products_by_supplier(
    *,
    db: Session = Depends(deps.get_db),
    supplier_id: str,
    skip: int = 0,
    limit: int = 100,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Retrieve products by supplier.
    """
    products = crud.product.get_multi_by_supplier(
        db, supplier_id=supplier_id, skip=skip, limit=limit
    )
    # Filter by owner
    return [p for p in products if p.owner_id == current_user.id]


@router.get("/low-stock", response_model=List[schemas.Product])
def read_low_stock_products(
    *,
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Retrieve products with stock levels below reorder level.
    """
    products = crud.product.get_low_stock_products(
        db, owner_id=current_user.id, skip=skip, limit=limit
    )
    return products


@router.get("/out-of-stock", response_model=List[schemas.Product])
def read_out_of_stock_products(
    *,
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
    current_user: models.User = Depends(deps.get_current_active_user),
) -> Any:
    """
    Retrieve products with zero stock.
    """
    products = crud.product.get_out_of_stock_products(
        db, owner_id=current_user.id, skip=skip, limit=limit
    )
    return products